"""Project settings.

There is no need to edit this file
unless you want to change values from the Kedro defaults.
For further information, including these default values, see
https://docs.kedro.org/en/stable/kedro_project_setup/settings.html.
"""

from kedro.config import OmegaConfigLoader

CONFIG_LOADER_CLASS = OmegaConfigLoader
CONFIG_LOADER_ARGS = {
    "base_env": "base",
    "default_run_env": "local",
}



import typing as t
from pathlib import Path

from .build_config import get_or_create_build_config, init_build_config
from .config_dirs import get_or_create_config_dirs, init_config_dir
from .modules import get_or_create_modules, init_module


def init_steps(project_root: Path) -> t.Generator[str, None, None]:
    yield "Looking for existing package directories"

    existing, build_config = get_or_create_build_config(project_root)
    if not existing:
        init_build_config(project_root, build_config=build_config)

    yield "Initialising config directories"

    config_dirs = get_or_create_config_dirs(project_root)
    for existing, target_config_dir in config_dirs.values():
        if not existing:
            init_config_dir(project_root, target_config_dir=target_config_dir)

    yield "Creating modules"

    modules = get_or_create_modules(project_root, build_config=build_config)
    for existing, target_module_path, module_contents_path in modules.values():
        if not existing:
            init_module(
                project_root,
                target_module_path=target_module_path,
                module_contents_path=module_contents_path,
            )


def init(project_root: Path) -> None:
    for _ in init_steps(project_root):
        pass



"""Project pipelines."""

from kedro.framework.project import find_pipelines
from kedro.pipeline import Pipeline


def register_pipelines() -> dict[str, Pipeline]:
    """Register the project's pipelines.

    Returns:
        A mapping from pipeline names to ``Pipeline`` objects.
    """
    pipelines = find_pipelines()
    # https://github.com/kedro-org/kedro/issues/2526
    pipelines["__default__"] = sum(pipelines.values(), start=Pipeline([]))
    return pipelines



import pytest
from kedro_init.modules import TEMPLATES_PATH, get_or_create_modules


@pytest.fixture
def project_setup(tmp_path):
    package_name = "test_package"
    src_dir = tmp_path / "src" / package_name
    src_dir.mkdir(parents=True)
    return tmp_path, package_name


def test_get_or_create_modules_gets_expected_result(project_setup):
    project_root, package_name = project_setup
    build_config = {"package_name": package_name}

    result = get_or_create_modules(project_root, build_config=build_config)

    assert result == {
        "pipeline_registry.py": (
            False,
            project_root / f"src/{package_name}/pipeline_registry.py",
            TEMPLATES_PATH / "pipeline_registry.py",
        ),
        "settings.py": (
            False,
            project_root / f"src/{package_name}/settings.py",
            TEMPLATES_PATH / "settings.py",
        ),
    }



from __future__ import annotations

from pathlib import Path

TEMPLATES_PATH = Path(__file__).parent / "templates"
MODULE_TEMPLATES = {
    "pipeline_registry.py": TEMPLATES_PATH / "pipeline_registry.py",
    "settings.py": TEMPLATES_PATH / "settings.py",
}


def get_or_create_modules(
    project_root: Path,
    *,
    build_config: dict[str, str],
    module_templates: dict[str, Path] | None = None,
) -> dict[str, tuple[bool, Path, Path]]:
    if module_templates:
        module_templates_l = module_templates
    else:
        module_templates_l = MODULE_TEMPLATES

    package_name = build_config["package_name"]
    package_dir = project_root / package_name
    if not package_dir.is_dir():
        package_dir = next(project_root.glob(f"*/{package_name}"), None)  # type: ignore

    if package_dir is None:
        raise ValueError(
            f"No suitable directory found for package name '{package_name}'"
        )

    modules = {}
    for module_name in module_templates_l:
        target_module = package_dir / module_name
        if target_module.exists():
            modules[module_name] = True, target_module, target_module
        else:
            modules[module_name] = False, target_module, module_templates_l[module_name]

    return modules


def init_module(
    project_root: Path, *, target_module_path: Path, module_contents_path: Path
) -> None:
    with target_module_path.open("w") as fh:
        fh.write(module_contents_path.read_text())



from __future__ import annotations

import tempfile
import typing as t
import zipfile
from pathlib import Path

from installer.sources import WheelFile
from installer.utils import parse_metadata_file

try:
    from importlib.metadata import PackageNotFoundError, version
except ModuleNotFoundError:
    from importlib_metadata import PackageNotFoundError, version  # type: ignore

import tomlkit
from pygetimportables import _simple_build_wheel, get_top_importables_from_wheel
from validate_pyproject import api, errors, plugins
from validate_pyproject.types import Schema


def _get_importables_and_project_name(
    project_root: str | Path, outdir: str | Path
) -> tuple[set[str], str]:
    wheel_path = _simple_build_wheel(project_root, outdir)

    with zipfile.ZipFile(wheel_path, "r") as zf:
        wheel_file = WheelFile(zf)
        metadata = parse_metadata_file(wheel_file.read_dist_info("METADATA"))

    package_names = get_top_importables_from_wheel(wheel_path)
    project_name = metadata["Name"]
    return package_names, project_name


def kedro_pyproject(tool_name: str) -> Schema:
    return Schema(
        {
            "$id": "https://docs.kedro.org/en/latest/",
            "type": "object",
            "description": "Kedro project metadata",
            "properties": {
                "package_name": {"type": "string"},
                "project_name": {"type": "string", "format": "pep508-identifier"},
                "kedro_init_version": {"type": "string", "format": "pep440"},
                "source_dir": {"type": "string"},
            },
            "required": ["package_name", "project_name", "kedro_init_version"],
            "additionalProperties": False,
        }
    )


def get_or_create_build_config(project_root: Path) -> tuple[bool, t.Any]:
    try:
        kedro_version = version("kedro")
    except PackageNotFoundError as exc:
        raise ValueError("Kedro is not installed") from exc

    available_plugins = [
        *plugins.list_from_entry_points(),
        plugins.PluginWrapper("kedro", kedro_pyproject),
    ]
    validator = api.Validator(available_plugins)

    with (project_root / "pyproject.toml").open("r") as fh:
        pyproject_toml = tomlkit.load(fh)

    if not pyproject_toml.get("tool", {}).get("kedro", {}):
        with tempfile.TemporaryDirectory() as outdir:
            package_names, project_name = _get_importables_and_project_name(
                project_root, outdir
            )
        if len(package_names) == 1:
            package_name = package_names.pop()
        else:
            raise ValueError("More than one package found in project root")

        kedro_config = {
            "project_name": project_name,
            "package_name": package_name,
            "kedro_init_version": kedro_version,
        }
        if (project_root / package_name).is_dir():
            kedro_config["source_dir"] = ""
        else:
            # FIXME: What happens if package_dir is None? (See type: ignore below)
            package_dir = next(project_root.glob(f"*/{package_name}"), None)
            source_dir = package_dir.parent.name if package_dir is not None else None
            if package_dir is not None and source_dir != "src":
                kedro_config["source_dir"] = source_dir  # type: ignore
        return False, kedro_config

    # Kedro build config might be present, return it if valid
    try:
        validator(pyproject_toml)
    except errors.ValidationError as exc:
        raise ValueError("Kedro build configuration is invalid") from exc
    else:
        return True, pyproject_toml["tool"]["kedro"]  # type: ignore


def init_build_config(project_root: Path, *, build_config: dict[str, str]) -> None:
    with (project_root / "pyproject.toml").open("r") as fh:
        pyproject_toml = tomlkit.load(fh)

    pyproject_toml.setdefault("tool", {})["kedro"] = build_config

    with (project_root / "pyproject.toml").open("w") as fh:
        tomlkit.dump(pyproject_toml, fh)



from pathlib import Path

import click

try:
    from rich.console import Console

    rich_available = True
except ImportError:
    rich_available = False

from .init import init, init_steps


@click.command()
@click.argument("project_root", type=click.Path(exists=False))
def cli(project_root: str) -> None:
    project_root_path = Path(project_root)
    if rich_available:
        console = Console()
        with console.status(
            f"Initialising Kedro project in {project_root_path.absolute()}"
        ):
            for step_message in init_steps(project_root_path):
                console.log(step_message)
            console.log(
                "[green]:large_orange_diamond: Kedro project successfully initialised!"
            )
    else:
        click.echo(f"Initialising Kedro project in {project_root_path.absolute()}...")
        init(project_root_path)
        click.echo("\U0001f536 Kedro project successfully initialised!")



from __future__ import annotations

from pathlib import Path

CONFIG_DIRS = [
    Path("conf") / "base",
    Path("conf") / "local",
]


def get_or_create_config_dirs(
    project_root: Path, *, expected_config_dirs: list[Path] | None = None
) -> dict[Path, tuple[bool, Path]]:
    if not expected_config_dirs:
        expected_config_dirs = CONFIG_DIRS

    config_dirs = {}
    for config_dir in expected_config_dirs:
        target_config_dir = project_root / config_dir
        if target_config_dir.is_dir():
            config_dirs[config_dir] = True, target_config_dir
        else:
            config_dirs[config_dir] = False, target_config_dir

    return config_dirs


def init_config_dir(project_root: Path, *, target_config_dir: Path) -> None:
    target_config_dir.mkdir(parents=True, exist_ok=True)
